@@ -1,19 +1,20 @@
 /*
     gwcontact.cpp - Kopete GroupWise Protocol
 
-    Copyright (c) 2004      SUSE Linux AG	 	 http://www.suse.com
+    Copyright (c) 2006      Novell, Inc	http://www.opensuse.org
+    Copyright (c) 2004      SUSE Linux AG http://www.suse.com
     
     Based on Testbed   
-    Copyright (c) 2003      by Will Stephenson		 <will@stevello.free-online.co.uk>
+    Copyright (c) 2003      by Will Stephenson <will@stevello.free-online.co.uk>
     
 	Blocking status taken from MSN
-    Copyright (c) 2003      by Will Stephenson		  <will@stevello.free-online.co.uk>
+    Copyright (c) 2003      by Will Stephenson <will@stevello.free-online.co.uk>
     Copyright (c) 2002      by Duncan Mac-Vicar Prett <duncan@kde.org>
-    Copyright (c) 2002      by Ryan Cumming           <bodnar42@phalynx.dhs.org>
-    Copyright (c) 2002-2003 by Martijn Klingens       <klingens@kde.org>
-    Copyright (c) 2002-2004 by Olivier Goffart        <ogoffart @ kde.org>
+    Copyright (c) 2002      by Ryan Cumming <bodnar42@phalynx.dhs.org>
+    Copyright (c) 2002-2003 by Martijn Klingens <klingens@kde.org>
+    Copyright (c) 2002-2004 by Olivier Goffart <ogoffart@kde.org>
     
-    Kopete    (c) 2002-2003 by the Kopete developers <kopete-devel@kde.org>
+    Kopete    (c) 2002-2007 by the Kopete developers <kopete-devel@kde.org>
 
     *************************************************************************
     *                                                                       *
@@ -25,14 +26,18 @@
     *************************************************************************
 */
 
+#include "gwcontact.h"
 #include <qmap.h>
 
 #include <kaction.h>
+#include <KActionCollection>
 #include <kdebug.h>
+#include <kicon.h>
 #include <klocale.h>
 
 #include <kopetemetacontact.h>
 #include <kopeteuiglobal.h>
+#include <kopeteprotocol.h>
 
 #include "client.h"
 #include "gwaccount.h"
@@ -42,17 +47,15 @@
 #include "tasks/updatecontacttask.h"
 #include "ui/gwcontactproperties.h"
 
-#include "gwcontact.h"
-
 using namespace GroupWise;
 
 GroupWiseContact::GroupWiseContact( Kopete::Account* account, const QString &dn, 
 			Kopete::MetaContact *parent, 
 			const int objectId, const int parentId, const int sequence )
 : Kopete::Contact( account, GroupWiseProtocol::dnToDotted( dn ), parent ), m_objectId( objectId ), m_parentId( parentId ),
-  m_sequence( sequence ), m_actionBlock( 0 ), m_archiving( false ), m_deleting( false )
+  m_sequence( sequence ), m_actionBlock( 0 ), m_archiving( false ), m_deleting( false ), m_messageReceivedOffline( false )
 {
-	if ( dn.find( '=' ) != -1 )
+	if ( dn.indexOf( '=' ) != -1 )
 	{
 		m_dn = dn;
 	}
@@ -63,11 +66,6 @@ GroupWiseContact::GroupWiseContact( Kopete::Account* account, const QString &dn,
 
 GroupWiseContact::~GroupWiseContact()
 {
-	// This is necessary because otherwise the userDetailsManager 
-	// would not fetch details for this contact if they contact you
-	// again from off-contact-list.
-	if ( metaContact()->isTemporary() )
-		account()->client()->userDetailsManager()->removeContact( contactId() );
 }
 
 QString GroupWiseContact::dn() const
@@ -77,38 +75,34 @@ QString GroupWiseContact::dn() const
 
 void GroupWiseContact::updateDetails( const ContactDetails & details )
 {
-	kdDebug( GROUPWISE_DEBUG_GLOBAL ) << k_funcinfo << endl;
+	kDebug() ;
 	if ( !details.cn.isNull() )
-		setProperty( protocol()->propCN, details.cn );
+		setProperty( protocol()->propCN, QVariant(details.cn) );
 	if ( !details.dn.isNull() )
 		m_dn = details.dn;
 	if ( !details.givenName.isNull() )
-		setProperty( protocol()->propGivenName, details.givenName );
+		setProperty( protocol()->propGivenName, QVariant(details.givenName) );
 	if ( !details.surname.isNull() )
-		setProperty( protocol()->propLastName, details.surname );
+		setProperty( protocol()->propLastName, QVariant(details.surname) );
 	if ( !details.fullName.isNull() )
-		setProperty( protocol()->propFullName, details.fullName );
+		setProperty( protocol()->propFullName, QVariant(details.fullName) );
 	m_archiving = details.archive;
 	if ( !details.awayMessage.isNull() )
-		setProperty( protocol()->propAwayMessage, details.awayMessage );
+		setStatusMessage( details.awayMessage );
 	
 	m_serverProperties = details.properties;
 	
-	QMap<QString, QString>::Iterator it;
 	// work phone number
-	if ( ( it = m_serverProperties.find( "telephoneNumber" ) )
-		!= m_serverProperties.end() )
-		setProperty( protocol()->propPhoneWork, it.data() );
+	if ( m_serverProperties.contains( "telephoneNumber" ) )
+		setProperty( protocol()->propPhoneWork, QVariant(m_serverProperties.value( "telephoneNumber" )) );
 	
 	// mobile phone number
-	if ( ( it = m_serverProperties.find( "mobile" ) )
-		!= m_serverProperties.end() )
-		setProperty( protocol()->propPhoneMobile, it.data() );
+	if ( m_serverProperties.contains( "mobile" ) )
+		setProperty( protocol()->propPhoneMobile, QVariant(m_serverProperties.value( "mobile" )) );
 	
 	// email
-	if ( ( it = m_serverProperties.find( "Internet EMail Address" ) )
-		!= m_serverProperties.end() )
-		setProperty( protocol()->propEmail, it.data() );
+	if ( m_serverProperties.contains( "Internet EMail Address" ) )
+		setProperty( protocol()->propEmail, QVariant(m_serverProperties.value( "Internet EMail Address" )) );
 		
 	if ( details.status != GroupWise::Invalid )
 	{	
@@ -134,7 +128,7 @@ bool GroupWiseContact::isReachable()
 	// in GWChatSession
 	// (This is a GroupWise rule, not a problem in Kopete)
 
-	if ( account()->isConnected() && isOnline()/* && account()->myself()->onlineStatus() != protocol()->groupwiseAppearOffline */)
+	if ( account()->isConnected() && ( isOnline() || messageReceivedOffline() ) /* && account()->myself()->onlineStatus() != protocol()->groupwiseAppearOffline */)
 		return true;
 	if ( !account()->isConnected()/* || account()->myself()->onlineStatus() == protocol()->groupwiseAppearOffline*/ )
 		return false;
@@ -150,47 +144,51 @@ void GroupWiseContact::serialize( QMap< QString, QString > &serializedData, QMap
 
 Kopete::ChatSession * GroupWiseContact::manager( Kopete::Contact::CanCreateFlags canCreate )
 {
-	//kdDebug( GROUPWISE_DEBUG_GLOBAL ) << k_funcinfo << "called, canCreate: " << canCreate << endl;
+	//kDebug() << "called, canCreate: " << canCreate;
 
 	Kopete::ContactPtrList chatMembers;
 	chatMembers.append( this );
 
-	return account()->chatSession( chatMembers, QString::null, canCreate );
+	return account()->chatSession( chatMembers, QString("")/*FIXME Check this is right*/, canCreate );
 }
 
-QPtrList<KAction> *GroupWiseContact::customContextMenuActions() 
+QList<KAction*> *GroupWiseContact::customContextMenuActions()
 {
-	QPtrList<KAction> *m_actionCollection = new QPtrList<KAction>;
+	QList<KAction *> * actions = new QList<KAction *>;
 
-	// Block/unblock Contact
+	// Block/unblock contact
 	QString label = account()->isContactBlocked( m_dn ) ? i18n( "Unblock User" ) : i18n( "Block User" );
 	if( !m_actionBlock )
 	{
-		m_actionBlock = new KAction( label, "msn_blocked",0, this, SLOT( slotBlock() ),
-			this, "actionBlock" );
+		m_actionBlock = new KAction( KIcon( "msn_blocked" ), label, 0 );
+		QObject::connect( m_actionBlock, SIGNAL( triggered( bool ) ), SLOT( slotBlock() ) );
 	}
 	else
 		m_actionBlock->setText( label );
 	m_actionBlock->setEnabled( account()->isConnected() );
-	
-	m_actionCollection->append( m_actionBlock );
 
-	return m_actionCollection;
+	actions->append( m_actionBlock );
+	// temporary action collection, used to apply Kiosk policy to the actions
+	KActionCollection tempCollection((QObject*)0);
+	tempCollection.addAction(QLatin1String("contactBlock"), m_actionBlock);
+
+	return actions;
 }
 
 void GroupWiseContact::slotUserInfo()
 {
-	new GroupWiseContactProperties( this, Kopete::UI::Global::mainWidget(), "gwcontactproperties" );
+	GroupWiseContactProperties *p = new GroupWiseContactProperties( this, Kopete::UI::Global::mainWidget() );
+	p->setObjectName( "gwcontactproperties" );
 }
 
-QMap< QString, QString > GroupWiseContact::serverProperties()
+QMap< QString, QVariant > GroupWiseContact::serverProperties()
 {
 	return m_serverProperties;
 }
 
 void GroupWiseContact::sendMessage( Kopete::Message &message )
 {
-	kdDebug( GROUPWISE_DEBUG_GLOBAL ) << k_funcinfo << endl;
+	kDebug() ;
 	manager()->appendMessage( message );
 	// tell the manager it was sent successfully
 	manager()->messageSucceeded();
@@ -199,6 +197,7 @@ void GroupWiseContact::sendMessage( Kopete::Message &message )
 void GroupWiseContact::deleteContact()
 {
 	account()->deleteContact( this );
+	Kopete::Contact::deleteContact();
 }
 
 void GroupWiseContact::sync( unsigned int)
@@ -208,7 +207,7 @@ void GroupWiseContact::sync( unsigned int)
 
 void GroupWiseContact::slotBlock()
 {
-	kdDebug( GROUPWISE_DEBUG_GLOBAL ) << k_funcinfo << endl;
+	kDebug() ;
 	if ( account()->isConnected() )
 	{
 		if ( account()->isContactBlocked( m_dn ) )
@@ -227,6 +226,7 @@ void GroupWiseContact::receivePrivacyChanged( const QString & dn, bool allow )
 
 void GroupWiseContact::setOnlineStatus( const Kopete::OnlineStatus& status )
 {
+	setMessageReceivedOffline( false );
 	if ( status == protocol()->groupwiseAwayIdle && status != onlineStatus() )
 		setIdleTime( 1 );
 	else if ( onlineStatus() == protocol()->groupwiseAwayIdle && status != onlineStatus() )
@@ -234,9 +234,9 @@ void GroupWiseContact::setOnlineStatus( const Kopete::OnlineStatus& status )
 	
 	if ( account()->isContactBlocked( m_dn ) && status.internalStatus() < 15 )
 	{
-		Kopete::Contact::setOnlineStatus(Kopete::OnlineStatus(status.status() , (status.weight()==0) ? 0 : (status.weight() -1)  ,
-			protocol() , status.internalStatus()+15 , QString::fromLatin1("msn_blocked"),
-			i18n("%1|Blocked").arg( status.description() ) ) );
+		Kopete::Contact::setOnlineStatus(Kopete::OnlineStatus(status.status(), (status.weight()==0) ? 0 : (status.weight() -1),
+			protocol() , status.internalStatus()+15 , QStringList("msn_blocked"),
+			i18n("%1|Blocked", status.description() ) ) );
 	}
 	else
 	{
@@ -297,8 +297,19 @@ void GroupWiseContact::renamedOnServer()
 			setProperty( Kopete::Global::Properties::self()->nickName(), uct->displayName() );
 	}
 	else
-		kdDebug( GROUPWISE_DEBUG_GLOBAL ) << k_funcinfo << "rename failed, return code: " << uct->statusCode() << endl;
+		kDebug() << "rename failed, return code: " << uct->statusCode();
 }
+
+void GroupWiseContact::setMessageReceivedOffline( bool on )
+{
+	m_messageReceivedOffline = on;
+}
+
+bool GroupWiseContact::messageReceivedOffline() const
+{
+	return m_messageReceivedOffline;
+}
+
 #include "gwcontact.moc"
 
 // vim: set noet ts=4 sts=4 sw=4:
