@@ -21,9 +21,12 @@
 #include "jabberbasecontact.h"
 #include "xmpp_vcard.h"
 
-#include "kopetemessagemanager.h" // needed for silly Kopete::ContactPtrList
+#include "kopetechatsession.h" // needed for silly Kopete::ContactPtrList
+
+#include <QList>
 
 class JabberChatSession;
+class QTimer;
 
 class JabberContact : public JabberBaseContact
 {
@@ -33,13 +36,16 @@ Q_OBJECT
 public:
 
 	JabberContact (const XMPP::RosterItem &rosterItem,
-				   JabberAccount *account, Kopete::MetaContact * mc);
+				   Kopete::Account *account, Kopete::MetaContact * mc, const QString &legacyId = QString());
+	
+	~JabberContact();
 
 	/**
 	 * Create custom context menu items for the contact
 	 * FIXME: implement manager version here?
 	 */
-	QPtrList<KAction> *customContextMenuActions ();
+	QList<KAction*> *customContextMenuActions ();
+	using JabberBaseContact::customContextMenuActions;
 
 	/**
 	 * Start a rename request.
@@ -59,11 +65,10 @@ public:
 	 */
 	Kopete::ChatSession *manager ( Kopete::Contact::CanCreateFlags );
 	
-	/**
-	 * Reads a vCard object and updates the contact's
-	 * properties accordingly.
-	 */
-	void setPropertiesFromVCard ( const XMPP::VCard &vCard );
+
+	virtual bool isContactRequestingEvent( XMPP::MsgEvent event );
+
+	virtual QString lastReceivedMessageId () const;
 
 public slots:
 
@@ -74,25 +79,50 @@ public slots:
 
 	/**
 	 * Sync Groups with server
+	 * 
+	 * operations are alctually performed in sloDelayedSync()
 	 */
 	void sync(unsigned int);
 
 	/**
 	 * This is the JabberContact level slot for sending files.
 	 *
-	 * @param sourceURL The actual KURL of the file you are sending
+	 * @param sourceURL The actual KUrl of the file you are sending
 	 * @param fileName (Optional) An alternate name for the file - what the
 	 *                 receiver will see
 	 * @param fileSize (Optional) Size of the file being sent. Used when sending
 	 *                 a nondeterminate file size (such as over a socket)
 	 */
-	virtual void sendFile( const KURL &sourceURL = KURL(),
-		const QString &fileName = QString::null, uint fileSize = 0L );
+	virtual void sendFile( const KUrl &sourceURL = KUrl(),
+		const QString &fileName = QString(), uint fileSize = 0L );
 
 	/**
-	 * Retrieve a vCard for the contact
+	 * Update the vCard on the server.
+	 * @todo is that still used ?
+	 */ 
+	void slotSendVCard();
+
+	/**
+	 * Set contact photo.
+	 * @param path Path to the photo.
 	 */
-	void slotUserInfo ();
+	void setPhoto(const QString &photoPath);
+	
+	/**
+	 * this will start a voice call to the contact
+	 */
+	//void voiceCall();
+	 
+	 /**
+	  * This is the JabberContact level slot for starting a Jingle Video call.
+	  * See later for params...
+	  */
+#ifdef JINGLE_SUPPORT
+	void showSessionsGui();
+	void startJingleSession();
+	void startJingleAudioCall();
+	void startJingleVideoCall();
+#endif
 
 private slots:
 
@@ -133,10 +163,10 @@ private slots:
 	 * Triggered as soon as Kopete changes its online state.
 	 */
 	void slotCheckVCard ();
-	
+
 	/**
 	 * Triggered from a timer, requests the vCard.
-	 * Timer is initiated by slotCheckVCard().
+	 * Timer is initiated by @ref slotCheckVCard.
 	 */
 	void slotGetTimedVCard ();
 	
@@ -145,6 +175,43 @@ private slots:
 	 */
 	void slotGotVCard ();
 
+	/**
+	 * Get information about last activity of the contact.
+	 * Triggered as soon as Kopete goes online or the contact goes offline.
+	 */
+	void slotCheckLastActivity ( Kopete::Contact *, const Kopete::OnlineStatus &, const Kopete::OnlineStatus & );
+
+	/**
+	 * Triggered from a timer, requests last activity information.
+	 * Timer is initiated by @ref slotCheckLastActivity.
+	 */
+	void slotGetTimedLastActivity ();
+
+	/**
+	 * Updates activity information.
+	 */
+	void slotGotLastActivity ();
+
+	/**
+	 * Display a error message if the vCard sent was unsuccessful.
+	 */
+	void slotSentVCard();
+	
+	/**
+	 * The service discovery on that contact is finished
+	 */
+	void slotDiscoFinished();
+	
+	/**
+	 * actually perform operations of sync() with a delay.
+	 * slot received by the syncTimer.
+	 */
+	void slotDelayedSync();
+
+#ifdef GOOGLETALK_SUPPORT
+	void makeGoogleTalkCallAction();
+#endif
+
 private:
 
 	/**
@@ -161,7 +228,7 @@ private:
 
 	/**
 	 * Create a message manager for this contact.
-	 * This version is suitable for group chat as it
+	 * This version is suitable for groupchat as it
 	 * looks for a message manager with a given
 	 * list of contacts as members.
 	 */
@@ -183,18 +250,31 @@ private:
 	 * resource name, if one account is interacting
 	 * with several resources of the same contact
 	 * at the same time. Note that this does *not*
-	 * apply to group chats, so this variable
+	 * apply to groupchats, so this variable
 	 * only contains classes of type JabberChatSession.
 	 * The casts in manager() and slotChatSessionDeleted()
 	 * are thus legal.
 	 */
-	QPtrList<JabberChatSession> mManagers;
+	QList<JabberChatSession*> mManagers;
 
 	/**
 	 * Indicates whether the vCard is currently
 	 * being updated or not.
 	 */
-	bool mVCardUpdateInProgress;
+	bool mVCardUpdateInProgress :1;
+
+	bool mRequestComposingEvent :1;
+	bool mRequestOfflineEvent :1;
+	bool mRequestDisplayedEvent :1;
+	bool mRequestDeliveredEvent :1;
+	bool mRequestGoneEvent :1;
+	/**
+	 * tell if the disco#info has been done for this contact.
+	 */
+	bool mDiscoDone :1;
+
+	QString mLastReceivedMessageId;
+	QTimer *m_syncTimer;
 
 };
 
