@@ -26,19 +26,17 @@
 #include "kopetepassword.h"
 
 #include <kaction.h>
-#include <kpopupmenu.h>
+#include <kactionmenu.h>
+#include <kmenu.h>
 #include <klocale.h>
 #include <kconfigbase.h>
-#include "kopeteaway.h"
 #include <kinputdialog.h>
 #include <kmessagebox.h>
-#include <qdict.h>
 
 MeanwhileAccount::MeanwhileAccount(
                         MeanwhileProtocol *parent,
-                        const QString &accountID,
-                        const char *name)
-    : Kopete::PasswordedAccount(parent, accountID, 0, name)
+                        const QString &accountID)
+    : Kopete::PasswordedAccount(parent, accountID)
 {
     HERE;
     m_meanwhileId = accountID;
@@ -106,6 +104,9 @@ void MeanwhileAccount::connectWithPassword(const QString &password)
         return;
     }
 
+   //     void connect(QString host, int port, QString account, QString password);
+
+
     if (!m_session->isConnected() && !m_session->isConnecting())
         m_session->connect(password);
 
@@ -131,19 +132,18 @@ void MeanwhileAccount::disconnect(Kopete::Account::DisconnectReason reason)
     m_session = 0L;
 }
 
-KActionMenu * MeanwhileAccount::actionMenu()
+void MeanwhileAccount::fillActionMenu( KActionMenu *actionMenu )
 {
-    KActionMenu *menu = Kopete::Account::actionMenu();
+	Kopete::Account::fillActionMenu( actionMenu );
 
-    menu->popupMenu()->insertSeparator();
+	actionMenu->menu()->addSeparator();
 
 #if 0
-    menu->insert(new KAction(i18n("&Change Status Message"), QString::null, 0,
+    actionMenu->insert(new KAction(i18n("&Change Status Message"), QString(), 0,
                 this, SLOT(meanwhileChangeStatus()), this,
                 "meanwhileChangeStatus"));
     //infoPlugin->addCustomMenus(theMenu);
 #endif
-    return menu;
 }
 
 QString MeanwhileAccount::getServerName()
@@ -153,7 +153,7 @@ QString MeanwhileAccount::getServerName()
 
 int MeanwhileAccount::getServerPort()
 {
-    return configGroup()->readNumEntry("Port");
+    return configGroup()->readEntry("Port").toInt();
 }
 
 void MeanwhileAccount::setServerName(const QString &server)
@@ -188,16 +188,24 @@ bool MeanwhileAccount::getClientIDParams(int *clientID,
     MeanwhileSession::getDefaultClientIDParams(clientID, verMajor, verMinor);
 
     if (custom_id) {
-	*clientID = configGroup()->readUnsignedNumEntry("clientID", *clientID);
-	*verMajor = configGroup()->readUnsignedNumEntry("clientVersionMajor",
-		    *verMinor);
-	*verMinor = configGroup()->readUnsignedNumEntry("clientVersionMinor",
-		    *verMinor);
+	*clientID = configGroup()->readEntry("clientID", *clientID);
+	*verMajor = configGroup()->readEntry("clientVersionMajor", *verMinor);
+	*verMinor = configGroup()->readEntry("clientVersionMinor", *verMinor);
     }
 
     return custom_id;
 }
 
+void MeanwhileAccount::setForceLogin(bool force)
+{
+    configGroup()->writeEntry("forceLogin", force);
+}
+
+bool MeanwhileAccount::getForceLogin()
+{
+    return configGroup()->readEntry("forceLogin", false);
+}
+
 void MeanwhileAccount::slotServerNotification(const QString &mesg)
 {
     KMessageBox::queuedMessageBox(0, KMessageBox::Error , mesg,
@@ -215,8 +223,13 @@ void MeanwhileAccount::setAway(bool away, const QString &reason)
     setOnlineStatus(away ? p->statusIdle : p->statusOnline, reason);
 }
 
+void MeanwhileAccount::setStatusMessage(const Kopete::StatusMessage &statusMessage)
+{
+    setOnlineStatus( myself()->onlineStatus(), statusMessage, Kopete::Account::KeepSpecialFlags );
+}
+
 void MeanwhileAccount::setOnlineStatus(const Kopete::OnlineStatus &status,
-        const QString &reason)
+                                       const Kopete::StatusMessage &reason, const OnlineStatusOptions& options)
 {
     HERE;
     Kopete::OnlineStatus oldstatus = myself()->onlineStatus();
@@ -261,7 +274,7 @@ void MeanwhileAccount::slotSessionStateChange(Kopete::OnlineStatus status)
 
     if (status.isDefinitelyOnline() != oldstatus.isDefinitelyOnline()) {
         if (status.isDefinitelyOnline())
-            m_session->addContacts(contacts());
+            m_session->addContacts(contacts()); // FIXME: what about myself??
         emit isConnectedChanged();
     }
 }
