@@ -20,11 +20,14 @@
 
 #include <qwidget.h>
 #include <qdatetime.h>
+#include <kurl.h>
 #include "kopetecontact.h"
 #include "kopetemessage.h"
 #include "userdetails.h"
 #include "client.h"
 #include "oscartypeclasses.h"
+#include "contact.h"
+#include "kopete_export.h"
 
 namespace Kopete
 {
@@ -32,10 +35,14 @@ class ChatSession;
 class OnlineStatus;
 }
 
+namespace Oscar
+{
+class Presence;
+}
+
 class OscarAccount;
-class QTimer;
 class QTextCodec;
-class KToggleAction;
+class OscarEncodingSelectionDialog;
 
 /**
  * Contact for oscar protocol
@@ -57,13 +64,13 @@ class KToggleAction;
  * \li get/set the ignore setting for the contact
  * \li get/set the visibility setting for the contact ( i.e. are we visible to the contact )
  */
-class KDE_EXPORT OscarContact : public Kopete::Contact
+class OSCAR_EXPORT OscarContact : public Kopete::Contact
 {
 Q_OBJECT
 
 public:
 	OscarContact( Kopete::Account* account, const QString& name,
-	              Kopete::MetaContact* parent, const QString& icon = QString::null, const Oscar::SSI& ssiItem = Oscar::SSI() );
+	              Kopete::MetaContact* parent, const QString& icon = QString() );
 	
 	virtual ~OscarContact();
 	
@@ -71,13 +78,13 @@ public:
 	
 	virtual Kopete::ChatSession *manager( Kopete::Contact::CanCreateFlags canCreate = Kopete::Contact::CanCreate );
 	
-	const QString &contactName() const { return mName; };
-	OscarAccount *account() const { return mAccount; };
+	const QString &contactName() const { return mName; }
+	OscarAccount *account() const { return mAccount; }
 	
 	bool isOnServer() const;
-	
-	void setSSIItem( const Oscar::SSI& item );
-	Oscar::SSI ssiItem() const;
+
+	virtual void setSSIItem( const OContact& item );
+	OContact ssiItem() const;
 	
 	/** we received a typing notification from this contact, tell any message manager */
 	void startedTyping();
@@ -89,10 +96,25 @@ public:
 	 */
 	QTextCodec *contactCodec() const;
 
-public slots:
-	/** slot so that properties can be updated based on a new SSI item */
-	virtual void updateSSIItem() = 0;
-	
+	/**
+	 * Returns true if contact has this capability
+	 */
+	bool hasCap( int capNumber ) const;
+
+	/**
+	 * Set presence target
+	 * Helper function that converts Oscar::Presence to Kopete::OnlineStatus and sets OnlineStatus.
+	 */
+	void setPresenceTarget( const Oscar::Presence &presence );
+
+	/**
+	 * Set encoding for this contact
+	 * @param mib the MIBenum
+	 * @note If @p mib is 0 then default encoding will be used
+	 */
+	virtual void setEncoding( int mib );
+
+public slots:	
 	/** Remove this contact from the server. Reimplemented from Kopete::Contact */
 	virtual void deleteContact();
 
@@ -108,32 +130,52 @@ public slots:
 	/** a user is offline */
 	virtual void userOffline( const QString& ) = 0;
 
-protected slots:
-	void slotTyping( bool typing );
-	virtual void updateFeatures() = 0;
+	/** send a file to this contact */
+	virtual void sendFile( const KUrl &sourceURL = KUrl(), const QString &fileName = QString(), uint fileSize = 0L );
+
+	/** set a away message */
+	virtual void setAwayMessage( const QString &message );
+
+	/** change contact encoding */
+	void changeContactEncoding();
+
+	void requestAuthorization();
 
 signals:
-	void updatedSSI();
-	void featuresUpdated();
-	
+	void statusMessageChanged();
+
+protected slots:
+	void slotTyping( bool typing );
+	void messageAck( const QString& contact, uint messageId );
+	void messageError( const QString& contact, uint messageId );
+
 protected:
 	OscarAccount *mAccount;
 	QString mName;
 	Kopete::ChatSession *mMsgManager;
 	UserDetails m_details;
-	SSI m_ssiItem;
+	OContact m_ssiItem;
 	int m_warningLevel;
 	QString m_clientFeatures;
-	
-private:
-	void initActions();
 
 protected slots:
-	virtual void slotSendMsg( Kopete::Message& msg, Kopete::ChatSession* session) = 0;
+	virtual void slotSendMsg( Kopete::Message& msg, Kopete::ChatSession* session);
 	
 private slots:
+	void changeEncodingDialogClosed( int );
 	void chatSessionDestroyed();
+	void requestBuddyIcon();
+	void haveIcon( const QString&, QByteArray );
+	void receivedStatusMessage( const QString& userId, const QString& message );
 	
+private:
+	QString filterAwayMessage( const QString &message ) const;
+	int oscarFontSize( int size ) const;
+	QString brMargin( int margin, int fontPointSize, bool forceBr = false ) const;
+
+	bool m_buddyIconDirty;
+
+	OscarEncodingSelectionDialog* m_oesd;
 };
 
 #endif
